# Git Large File Storage

<img src="logo.svg" width="100px" />

*How to use Git LFS on JLU GitLab (and if this is a good idea)*

__NOTE: This project is work in progress!__

[[_TOC_]]


## What problem is solved by Git LFS?
The main purpose of Git LFS is to treat **data** files *as conveniently as if they were inside* a Git repository, while *actually keeping them outside* of the repository.
There are technical reasons in Git's design to make this extra step necessary:

- Large files will bloat the Git repository for everyone who has a clone, and degrade the performance of Git operations.
  Git is optimized for text-based content, not for binary files.
- Git is designed to distribute the entire snapshot history to every clone.
  If data are part of the Git repository (at any point in history!), it means it gets replicated on every clone, even if's not needed (any more).
- Git is designed to make it impossible to delete data from the repository's history.
  All you can do to "delete data" is force Git to you explicitly re-write the snapshot history.
  Even if you work alone, this is a bit of a hassle — it gets much worse if other people have clones, because history change requires everyone involved to confirm the deletion with their clone.


## Is it a good idea to use Git LFS for your project?
You must consider several aspects before uploading any data to JLU GitLab.
Please read [this information](https://gitlab.test.uni-giessen.de/jlugitlab/support/-/blob/master/en/Information.md#storage-of-research-data) on research data management.
If in doubt about your specific situation, please consult the department for research data, [forschungsdaten@uni-giessen.de](mailto:forschungsdaten@uni-giessen.de).

## Practical steps how to use Git LFS
Assumptions:
- You have Git installed on your machine and you know the basics how to use it (TODO, specify: `add`, `commit`, `pull`, `push`).
  (If you don't, [here is a good starting point](https://git-scm.com/)).
- You have a project on JLU GitLab that includes a Git repository, and you have a local clone of it on your machine.
- You can type Git commands into a command line interface (terminal).
  Below, example terminal commands are indicated with a different font and with a leading dollar sign, `$ like this`.
- You have the Git LFS extension installed on your machine (you can find [instructions here](https://git-lfs.github.com/)).

1. In your local repository clone, configure which types of files you want to track by LFS.
    - For example, to let LFS keep track of `CSV` files, type: `$ git lfs track "*.csv"`.
    - This will create/change the Git configuration file [`.gitattributes`](.gitattributes).
      You should track this configuration change in Git, e.g. by the usual Git commands `$ git add .gitattributes` and `$ git commit -m "start tracking HDF files with LFS"`.  
      *Note that because the file name `.gitattributes` starts with a dot, it may be hidden from view (on Linux and MacOS, use `$ ls -a` to see it; FIXME: What to do on Windows?).*
2. FIXME: How to generally add files into LFS, and how to interact with them.
3. FIXME: Clarify the locking mechanism.


## Example(s)
- [Here](example) you can find an of an analysis script that relies on data stored in Git LFS.
- TODO: Maybe add more examples with different data (audio, video, images)?


## Useful links
- https://git-lfs.github.com/
- https://docs.gitlab.com/ce/topics/git/lfs/
