# Example: Analysis that relies on data in Git LFS

## Main illustration
This example illustrates how Git LFS enables tight integration of analysis code and its data:
Researchers who want to reproduce the results just need to clone this repository to get both.

The example consists of
- an analysis script ([example_analyis_script.ipynb](example_analyis_script.ipynb)),
- which relies on binary data ([example_data.hdf](example_data.hdf)) stored by Git LFS,
- and stores figure data (as `.png` files) in the folder [plots](plots), stored by Git LFS.

The analysis script and this README file are versioned in normal Git snapshots (without LFS).
In GitLab's file overview, the small "LFS" badge next to the file names visualizes which files are stored in LFS.
If you cloned the repository on your machine (and you have LFS installed), you type `$ git lfs ls-files` to get an overview of which files are stored in Git LFS.
In this example, you would see something like this:
```
$ git lfs ls-files

3cf63089c8 * example/example_data.hdf
3d177cad03 * example/plots/plot_example_cumsum.png
f4aa57ea83 * example/plots/plot_example_histogram.png
82065f2929 * example/plots/plot_example_trace.png
```

As a nice touch, you can even link and embed files in LFS just as if they were in the repository itself.
For example, to embed one of the plots, use `![](plots/plot_example_trace.png)`:

![](plots/plot_example_trace.png)


## How to reproduce this setup?
To achieve tracking with LFS of these files types (PNG and HDF), irrespective of their path in the repository, you can use these two commands:

```sh
$ git lfs track "*.hdf"
$ git lfs track "*.png"
```

This should result in a Git configuration file `.gitattributes` that contains these lines:

```
*.hdf filter=lfs diff=lfs merge=lfs -text
*.png filter=lfs diff=lfs merge=lfs -text
```

## Appendix information
- For the sake of readibility in the browser, the analysis script is a [Jupyter notebook](https://jupyter.org/).
- The data are stored in the open data file format [HDF-5](https://www.hdfgroup.org/).
